'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var qs = _interopDefault(require('query-string'));
var toRegex = _interopDefault(require('path-to-regexp'));
var createHistory = _interopDefault(require('history/createBrowserHistory'));

function matchURI(pattern, keys, ref) {
  var pathname = ref.pathname;
  var search = ref.search;

  var match = pattern.exec(pathname);
  if (!match) { return null; }
  var params = {};

  for (var i = 1; i < match.length; i++) {
    params[keys[i - 1].name] = match[i] !== undefined ? match[i] : undefined;
  }

  return [Object.assign({}, params, qs.parse(search)), match[0]];
}

function compilePath(path, ref) {
  var exact = ref.exact;

  var keys = [];

  var regex = toRegex(path, keys, { end: !!exact });

  return [regex, keys];
}

var cache = {};

function matchPath(path, ref, context) {
  var exact = ref.exact; if ( exact === void 0 ) exact = false;

  if (!path) { return []; }

  var key = "" + path + (exact.toString());

  var ref$1 = cache[key] || compilePath(path, { exact: exact });
  var regex = ref$1[0];
  var keys = ref$1[1];

  cache[key] = [regex, keys];

  return matchURI(regex, keys, context);
}

var MatchFirst = {
  name: 'MatchFirst',
  inject: ['router'],
  render: function render(h) {
    var this$1 = this;

    var vnode = this.$slots.default.find(function (vnode) {
      if (!vnode.componentOptions) { return false; }

      var ref = vnode.componentOptions.propsData;
      var path = ref.path;
      var exact = ref.exact;

      return matchPath(path, { exact: exact }, this$1.router.location);
    });

    return vnode;
  }
};

var Redirect = {
  name: 'Redirect',
  inject: ['router'],
  props: {
    to: {
      type: String,
      required: true
    }
  },
  created: function created() {
    this.router.history.replace(this.to);
  },
  render: function render() {
    return null;
  }
};

function injectProps(component, props) {
  var options = component.componentOptions = component.componentOptions || {};
  var propsData = options.propsData = options.propsData || {};

  Object.assign(propsData, props);
}

var Route = {
  name: 'Route',
  inject: ['router'],
  props: {
    path: {
      type: String,
      default: ''
    },
    exact: {
      type: Boolean,
      default: false
    }
  },
  render: function render(h) {
    var match = matchPath(this.path, { exact: !!this.exact }, this.router.location);

    var ref = match || [];
    var props = ref[0];
    var path = ref[1];

    var params = Object.assign({}, props, { url: path, path: this.path, router: this.router });

    if (this.$scopedSlots.children) {
      return this.$scopedSlots.children(params);
    }

    if (match) {
      if (this.$scopedSlots.default) {
        return this.$scopedSlots.default(params)[0];
      }

      injectProps(this.$slots.default[0], params);

      return this.$slots.default[0];
    }
  }
};

var RouterLink = {
  name: 'RouterLink',
  inject: ['router'],
  props: {
    to: {
      type: String,
      required: true
    },
    activeClass: {
      type: String,
      default: 'active'
    }
  },
  methods: {
    handleClick: function handleClick(e) {
      e.preventDefault();

      this.router.history.push(this.to);
    }
  },
  render: function render(h) {
    var obj;

    var href = this.to;

    var match = matchPath(this.to, { exact: false }, this.router.location);

    return h('a', {
      class: ( obj = {}, obj[this.activeClass] = !!match, obj),
      on: {
        click: this.handleClick
      },
      attrs: {
        href: href
      }
    }, this.$slots.default);
  }
};

var history = createHistory();

var Router = {
  name: 'Router',
  props: {
    history: {
      type: Object,
      required: true
    },
    location: {
      type: Object,
      required: true
    }
  },
  provide: function provide() {
    return {
      router: {
        location: this.location,
        history: this.history
      }
    };
  },
  render: function render(h) {
    return this.$slots.default[0];
  }
};

var HistoryRouter = {
  name: 'HistoryRouter',
  data: function data() {
    return {
      history: history,
      location: history.location
    };
  },
  created: function created() {
    var this$1 = this;

    var onLocationChange = function (location) {
      this$1.$emit('locationChanged', location);
      Object.assign(this$1.location, location);
    };

    this.history.listen(onLocationChange);
  },
  render: function render(h) {
    return h(Router, {
      props: {
        history: this.history,
        location: this.location
      }
    }, this.$slots.default);
  }
};

var withRouter = function (Component) {
  var inject = Component.inject || [];

  inject.push('router');

  Component.inject = inject;

  return Component;
};

exports.HistoryRouter = HistoryRouter;
exports.Router = Router;
exports.matchPath = matchPath;
exports.MatchFirst = MatchFirst;
exports.Redirect = Redirect;
exports.Route = Route;
exports.RouterLink = RouterLink;
exports.withRouter = withRouter;
